(function ($, document, window, viewport) {
    'use strict';

    // PLUGIN CLASS
    // ============
    const LineClampPlugin = {
        DEFAULTS: {
            "target": null,
            "maxHeight": {
                "xs": 0,
                "sm": 0,
                "md": 0,
                "lg": 0,
                "xl": 0
            },
            "triggerMore": ".et-line-clamp-more",
            "triggerLess": ".et-line-clamp-less"
        },

        /** @type {HTMLElement} */
        element: null,

        /** @type {JQuery<HTMLElement>} */
        $target: null,

        /** @type {JQuery<HTMLElement>} */
        $triggerMore: null,

        /** @type {JQuery<HTMLElement>} */
        $triggerLess: null,

        /**
         * @this {PluginBase & LineClampPlugin}
         * @param {HTMLElement} element
         * @param {Object} options
         */
        init: function (element, options) {
            this.element = element;
            this.options = $.extend(this.options, {
                "target": this.$el.data('line-clamp-target'),
                "maxHeight": this.$el.data('line-clamp'),
                "triggerMore": this.$el.data('line-clamp-more'),
                "triggerLess": this.$el.data('line-clamp-less')
            });

            if (this.options.target) {
                let $foundTarget = this.$el.parent().find(this.options.target);
                if ($foundTarget.length) {
                    this.$target = $foundTarget;
                } else {
                    this.$target = this.$el;
                }
            } else {
                this.$target = this.$el;
            }

            this.$triggerMore = this.$el.parent().find(this.options.triggerMore).first();
            this.$triggerLess = this.$el.parent().find(this.options.triggerLess).first();
            this.clampLines();
        },

        /**
         * Calculate number of lines based on the max height of the content to clamp appropriatly.
         *
         * @this {PluginBase & LineClampPlugin}
         */
        clampLines: function () {
            const maxHeight = this.options.maxHeight[viewport.current()] || 0;
            if (maxHeight <= 0) {
                return;
            }

            this.$target.each((index, el) => {
                const $el = $(el);
                $el.addClass('initializing');

                const lines = Math.floor(maxHeight / parseFloat($el.css('line-height')));

                const clampProperties = {
                    'line-clamp': lines.toString(), // lines have to be a string, otherwise it will be ignored!
                    'overflow': 'hidden',
                    'display': '-webkit-box',
                    '-webkit-box-orient': 'vertical'
                }

                const resetProperties = {
                    'line-clamp': 'initial',
                    'overflow': $el.css('overflow'),
                    'display': $el.css('display'),
                    '-webkit-box-orient': $el.css('-webkit-box-orient')
                }
                // line clamp the element
                $el.css(clampProperties);

                // detect if element is now actually line-clamped and show line-clamp-more trigger
                if ($el[0].offsetHeight < $el[0].scrollHeight) {
                    this.$triggerMore.show();
                    this.$triggerLess.hide();

                    this.$triggerMore.on('click', (event) => {
                        event.preventDefault();
                        $el.css(resetProperties);
                        this.$triggerMore.hide();
                        this.$triggerLess.show();
                    });

                    this.$triggerLess.on('click', (event) => {
                        event.preventDefault();
                        $el.css(clampProperties);
                        this.$triggerMore.show();
                        this.$triggerLess.hide();
                    });
                }

                $el.addClass('initialized').removeClass('initializing');
            });

            this.Events.publish('easytemplate/Plugin/LineClampPlugin/onClampLines', [this, {maxHeight}]);
        }
    };

    // REGISTER AND ADD THE PLUGIN
    // ===========================
    if (!window.easytemplate && !window.easytemplate.PluginManager) {
        throw Error('Cannot initialize Easytemplate Plugin. plugin-manager.js has to be loaded first!');
    }

    /** @type {PluginManager} manager */
    const manager = window.easytemplate.PluginManager;
    manager.addPlugin('[data-line-clamp]', LineClampPlugin, 'etLineClampPlugin');
})(jQuery, document, window, ResponsiveBootstrapToolkit);